@@ -2,41 +2,37 @@
 Custom Qt widgets that serve as native objects that the public-facing elements
 wrap.
 """
+import os.path
 import time
 
-# set vispy to use same backend as qtpy
-from ..utils.io import imsave
-
-from .qt_viewer import QtViewer
-from .qt_about import QtAbout
-from .qt_plugin_report import QtPluginErrReporter
-from .qt_plugin_sorter import QtPluginSorter
-from .qt_plugin_table import QtPluginTable
-from .qt_debug_menu import DebugMenu
-
-from .qt_viewer_dock_widget import QtViewerDockWidget
-from ..resources import get_stylesheet
-from ..utils import perf
-
-# these "# noqa" comments are here to skip flake8 linting (E402),
-# these module-level imports have to come after `app.use_app(API)`
-# see discussion on #638
-from qtpy.QtWidgets import (  # noqa: E402
-    QApplication,
-    QMainWindow,
-    QWidget,
-    QHBoxLayout,
-    QDockWidget,
-    QLabel,
+from qtpy.QtCore import Qt
+from qtpy.QtGui import QIcon, QKeySequence
+from qtpy.QtWidgets import (
     QAction,
+    QApplication,
+    QDockWidget,
+    QHBoxLayout,
+    QLabel,
+    QMainWindow,
     QShortcut,
     QStatusBar,
-    QFileDialog,
+    QWidget,
 )
-from qtpy.QtCore import Qt  # noqa: E402
-from qtpy.QtGui import QKeySequence, QIcon  # noqa: E402
-from .utils import QImg2array  # noqa: E402
-from ..utils.theme import template  # noqa: E402
+
+from ..resources import get_stylesheet
+from ..utils import perf
+from ..utils.io import imsave
+from ..utils.misc import in_jupyter
+from ..utils.theme import template
+from .dialogs.qt_about import QtAbout
+from .dialogs.qt_plugin_report import QtPluginErrReporter
+from .dialogs.qt_plugin_table import QtPluginTable
+from .dialogs.screenshot_dialog import ScreenshotDialog
+from .qt_viewer import QtViewer
+from .tracing.qt_debug_menu import DebugMenu
+from .utils import QImg2array
+from .widgets.qt_plugin_sorter import QtPluginSorter
+from .widgets.qt_viewer_dock_widget import QtViewerDockWidget
 
 
 class Window:
@@ -222,6 +218,11 @@ class Window:
                 QApplication.setWindowIcon(QIcon())
                 self.close()
 
+            if perf.USE_PERFMON:
+                # Write trace file before exit, if we were writing one.
+                # Is there a better place to make sure this is done on exit?
+                perf.timers.stop_trace_file()
+
         exitAction.triggered.connect(handle_exit)
 
         self.file_menu = self.main_menu.addMenu('&File')
@@ -270,6 +271,12 @@ class Window:
         list_plugins_action.triggered.connect(self._show_plugin_list)
         self.plugins_menu.addAction(list_plugins_action)
 
+        pip_install_action = QAction(
+            "Install/Uninstall Package(s)...", self._qt_window
+        )
+        pip_install_action.triggered.connect(self._show_pip_install_dialog)
+        self.plugins_menu.addAction(pip_install_action)
+
         order_plugin_action = QAction("Plugin Call Order...", self._qt_window)
         order_plugin_action.setStatusTip('Change call order for plugins')
         order_plugin_action.triggered.connect(self._show_plugin_sorter)
@@ -296,6 +303,13 @@ class Window:
         plugin_sorter.finished.connect(plugin_sorter.deleteLater)
         plugin_sorter.finished.connect(dock_widget.deleteLater)
 
+    def _show_pip_install_dialog(self):
+        """Show dialog that allows users to sort the call order of plugins."""
+        from .qt_pip_dialog import QtPipDialog
+
+        dialog = QtPipDialog(self._qt_window)
+        dialog.exec_()
+
     def _show_plugin_err_reporter(self):
         """Show dialog that allows users to review and report plugin errors."""
         plugin_sorter = QtPluginErrReporter(parent=self._qt_window)
@@ -416,17 +430,24 @@ class Window:
         # Resize axis labels now that window is shown
         self.qt_viewer.dims._resize_axis_labels()
 
-        # We want to call Window._qt_window.raise_() in every case *except*
-        # when instantiating a viewer within a gui_qt() context for the
-        # _first_ time within the Qt app's lifecycle.
-        #
+        # We want to bring the viewer to the front when
+        # A) it is our own (gui_qt) event loop OR we are running in jupyter
+        # B) it is not the first time a QMainWindow is being created
+
         # `app_name` will be "napari" iff the application was instantiated in
         # gui_qt(). isActiveWindow() will be True if it is the second time a
-        # _qt_window has been created. See #732
+        # _qt_window has been created.
+        # See #721, #732, #735, #795, #1594
         app_name = QApplication.instance().applicationName()
-        if app_name != 'napari' or self._qt_window.isActiveWindow():
-            self._qt_window.raise_()  # for macOS
-            self._qt_window.activateWindow()  # for Windows
+        if (
+            app_name == 'napari' or in_jupyter()
+        ) and self._qt_window.isActiveWindow():
+            self.activate()
+
+    def activate(self):
+        """Make the viewer the currently active window."""
+        self._qt_window.raise_()  # for macOS
+        self._qt_window.activateWindow()  # for Windows
 
     def _update_palette(self, event=None):
         """Update widget color palette."""
@@ -477,21 +498,11 @@ class Window:
 
     def _screenshot_dialog(self):
         """Save screenshot of current display with viewer, default .png"""
-        filename, _ = QFileDialog.getSaveFileName(
-            parent=self.qt_viewer,
-            caption='Save screenshot with viewer',
-            directory=self.qt_viewer._last_visited_dir,  # home dir by default
-            filter="Image files (*.png *.bmp *.gif *.tif *.tiff)",  # first one used by default
-            # jpg and jpeg not included as they don't support an alpha channel
+        dial = ScreenshotDialog(
+            self.screenshot, self.qt_viewer, self.qt_viewer._last_visited_dir
         )
-        if (filename != '') and (filename is not None):
-            # double check that an appropriate extension has been added as the
-            # filter option does not always add an extension on linux and windows
-            # see https://bugreports.qt.io/browse/QTBUG-27186
-            image_extensions = ('.bmp', '.gif', '.png', '.tif', '.tiff')
-            if not filename.endswith(image_extensions):
-                filename = filename + '.png'
-            self.screenshot(path=filename)
+        if dial.exec_():
+            self._last_visited_dir = os.path.dirname(dial.selectedFiles()[0])
 
     def screenshot(self, path=None):
         """Take currently displayed viewer and convert to an image array.
@@ -514,7 +525,16 @@ class Window:
 
     def close(self):
         """Close the viewer window and cleanup sub-widgets."""
-        # on some versions of Darwin, exiting while fullscreen seems to tickle
+
+        # Someone is closing us twice? Only try to delete self._qt_window
+        # if we still have one.
+        if hasattr(self, '_qt_window'):
+            self._delete_qt_window()
+
+    def _delete_qt_window(self):
+        """Delete our self._qt_window."""
+
+        # On some versions of Darwin, exiting while fullscreen seems to tickle
         # some bug deep in NSWindow.  This forces the fullscreen keybinding
         # test to complete its draw cycle, then pop back out of fullscreen.
         if self._qt_window.isFullScreen():
