@@ -1,8 +1,10 @@
 import numpy as np
 import pytest
-from vispy.color import Colormap
 
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Labels
+from napari.layers.labels._labels_constants import LabelColorMode
+from napari.utils import Colormap
 
 
 def test_random_labels():
@@ -14,7 +16,6 @@ def test_random_labels():
     assert np.all(layer.data == data)
     assert layer.ndim == len(shape)
     assert layer.shape == shape
-    assert layer.dims.range == [(0, m, 1) for m in shape]
     assert layer._data_view.shape == shape[-2:]
     assert layer.editable is True
 
@@ -42,7 +43,7 @@ def test_3D_labels():
     assert layer._data_view.shape == shape[-2:]
     assert layer.editable is True
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer.dims.ndisplay == 3
     assert layer.editable is False
     assert layer.mode == 'pan_zoom'
@@ -60,7 +61,6 @@ def test_changing_labels():
     assert np.all(layer.data == data_b)
     assert layer.ndim == len(shape_b)
     assert layer.shape == shape_b
-    assert layer.dims.range == [(0, m, 1) for m in shape_b]
     assert layer._data_view.shape == shape_b[-2:]
 
 
@@ -77,7 +77,6 @@ def test_changing_labels_dims():
     assert np.all(layer.data == data_b)
     assert layer.ndim == len(shape_b)
     assert layer.shape == shape_b
-    assert layer.dims.range == [(0, m, 1) for m in shape_b]
     assert layer._data_view.shape == shape_b[-2:]
 
 
@@ -278,26 +277,28 @@ def test_colormap():
     np.random.seed(0)
     data = np.random.randint(20, size=(10, 15))
     layer = Labels(data)
-    assert type(layer.colormap) == tuple
-    assert layer.colormap[0] == 'random'
-    assert type(layer.colormap[1]) == Colormap
+    assert isinstance(layer.colormap, Colormap)
+    assert layer.colormap.name == 'label_colormap'
 
     layer.new_colormap()
-    assert type(layer.colormap) == tuple
-    assert layer.colormap[0] == 'random'
-    assert type(layer.colormap[1]) == Colormap
+    assert isinstance(layer.colormap, Colormap)
+    assert layer.colormap.name == 'label_colormap'
 
 
 def test_custom_color_dict():
     """Test custom color dict."""
     np.random.seed(0)
     data = np.random.randint(20, size=(10, 15))
-    layer = Labels(data, color={1: 'white'})
+    layer = Labels(
+        data, color={2: 'white', 4: 'red', 8: 'blue', 16: 'red', 32: 'blue'}
+    )
 
     # test with custom color dict
     assert type(layer.get_color(2)) == np.ndarray
     assert type(layer.get_color(1)) == np.ndarray
-    assert (layer.get_color(1) == np.array([1.0, 1.0, 1.0, 1.0])).all()
+    assert (layer.get_color(2) == np.array([1.0, 1.0, 1.0, 1.0])).all()
+    assert (layer.get_color(4) == layer.get_color(16)).all()
+    assert (layer.get_color(8) == layer.get_color(32)).all()
 
     # test disable custom color dict
     # should not initialize as white since we are using random.seed
@@ -374,6 +375,33 @@ def test_label_color():
     assert len(col) == 4
 
 
+def test_selected_mode_label_color():
+    """Test color of labels in selected color mode"""
+    np.random.seed(0)
+    data = np.random.randint(20, size=(10, 15))
+    layer = Labels(data)
+    original_color = layer.get_color(1)
+
+    layer.color_mode = LabelColorMode.SELECTED
+    original_background_color = layer.get_color(layer._background_label)
+    none_color = layer.get_color(None)
+    layer.selected_label = 1
+
+    # color of selected label has not changed
+    assert np.allclose(layer.get_color(layer.selected_label), original_color)
+
+    current_background_color = layer.get_color(layer._background_label)
+    # color of background is background color
+    assert current_background_color == original_background_color
+
+    # color of all others is none color
+    other_labels = np.unique(layer.data)[2:]
+    other_colors = np.array(
+        list(map(lambda x: layer.get_color(x), other_labels))
+    )
+    assert np.allclose(other_colors, none_color)
+
+
 def test_paint():
     """Test painting labels with different square brush sizes."""
     np.random.seed(0)
@@ -519,3 +547,13 @@ def test_thumbnail():
     layer = Labels(data)
     layer._update_thumbnail()
     assert layer.thumbnail.shape == layer._thumbnail_shape
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    np.random.seed(0)
+    shape = (6, 10, 15)
+    data = np.random.randint(20, size=(shape))
+    layer = Labels(data)
+    extent = np.array(((0,) * 3, shape))
+    check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5))
