@@ -4,8 +4,12 @@ from typing import Dict, Union
 import numpy as np
 from scipy import ndimage as ndi
 
-from ...utils.colormaps import colormaps
-from ...utils.event import Event
+from ...utils.colormaps import (
+    color_dict_to_colormap,
+    label_colormap,
+    low_discrepancy_image,
+)
+from ...utils.events import Event
 from ...utils.status_messages import format_float
 from ..image import Image
 from ..utils.color_transformations import transform_color
@@ -148,10 +152,7 @@ class Labels(Image):
         self._seed = seed
         self._background_label = 0
         self._num_colors = num_colors
-        self._random_colormap = (
-            'random',
-            colormaps.label_colormap(self.num_colors),
-        )
+        self._random_colormap = label_colormap(self.num_colors)
         self._color_mode = LabelColorMode.AUTO
         self._brush_shape = LabelBrushShape.CIRCLE
 
@@ -221,7 +222,7 @@ class Labels(Image):
 
         self.dims.events.ndisplay.connect(self._reset_history)
         self.dims.events.order.connect(self._reset_history)
-        self.dims.events.axis.connect(self._reset_history)
+        self.dims.events.current_step.connect(self._reset_history)
 
     @property
     def contiguous(self):
@@ -275,10 +276,7 @@ class Labels(Image):
     @num_colors.setter
     def num_colors(self, num_colors):
         self._num_colors = num_colors
-        self.colormap = (
-            self._colormap_name,
-            colormaps.label_colormap(num_colors),
-        )
+        self.colormap = label_colormap(num_colors)
         self.refresh()
         self._selected_color = self.get_color(self.selected_label)
         self.events.selected_label()
@@ -384,6 +382,11 @@ class Labels(Image):
         self._selected_color = self.get_color(selected_label)
         self.events.selected_label()
 
+        # note: self.color_mode returns a string and this comparison fails,
+        # so use self._color_mode
+        if self._color_mode == LabelColorMode.SELECTED:
+            self.refresh()
+
     @property
     def color_mode(self):
         """Color mode to change how color is represented.
@@ -391,6 +394,8 @@ class Labels(Image):
         AUTO (default) allows color to be set via a hash function with a seed.
 
         DIRECT allows color of each label to be set directly by a color dict.
+
+        SELECTED allows only selected labels to be visible.
         """
         return str(self._color_mode)
 
@@ -398,15 +403,16 @@ class Labels(Image):
     def color_mode(self, color_mode: Union[str, LabelColorMode]):
         color_mode = LabelColorMode(color_mode)
         if color_mode == LabelColorMode.DIRECT:
-            (
-                custom_colormap,
-                label_color_index,
-            ) = colormaps.color_dict_to_colormap(self.color)
+            (custom_colormap, label_color_index,) = color_dict_to_colormap(
+                self.color
+            )
             self.colormap = custom_colormap
             self._label_color_index = label_color_index
         elif color_mode == LabelColorMode.AUTO:
             self._label_color_index = {}
             self.colormap = self._random_colormap
+        elif color_mode == LabelColorMode.SELECTED:
+            pass
         else:
             raise ValueError("Unsupported Color Mode")
 
@@ -565,8 +571,30 @@ class Labels(Image):
             )[inv].reshape(raw.shape)
         elif self._color_mode == LabelColorMode.AUTO:
             image = np.where(
-                raw > 0, colormaps._low_discrepancy_image(raw, self._seed), 0
+                raw > 0, low_discrepancy_image(raw, self._seed), 0
             )
+        elif self._color_mode == LabelColorMode.SELECTED:
+            selected = self._selected_label
+            # we were in direct mode previously
+            if self._label_color_index:
+                if selected not in self._label_color_index:
+                    selected = None
+                index = self._label_color_index
+                image = np.where(
+                    raw == selected,
+                    index[selected],
+                    np.where(
+                        raw != self._background_label,
+                        index[None],
+                        index[self._background_label],
+                    ),
+                )
+            else:
+                image = np.where(
+                    raw == selected,
+                    low_discrepancy_image(selected, self._seed),
+                    0,
+                )
         else:
             raise ValueError("Unsupported Color Mode")
         return image
@@ -580,7 +608,7 @@ class Labels(Image):
             col = None
         else:
             val = self._raw_to_displayed(np.array([label]))
-            col = self.colormap[1][val].rgba[0]
+            col = self.colormap.map(val)[0]
         return col
 
     def _reset_history(self, event=None):
@@ -597,7 +625,7 @@ class Labels(Image):
     def _save_history(self):
         self._redo_history = deque()
         if not self._block_saving:
-            self._undo_history.append(self.data[self.dims.indices].copy())
+            self._undo_history.append(self.data[self._slice_indices].copy())
             self._trim_history()
 
     def _load_history(self, before, after):
@@ -605,8 +633,8 @@ class Labels(Image):
             return
 
         prev = before.pop()
-        after.append(self.data[self.dims.indices].copy())
-        self.data[self.dims.indices] = prev
+        after.append(self.data[self._slice_indices].copy())
+        self.data[self._slice_indices] = prev
 
         self.refresh()
 
@@ -636,7 +664,7 @@ class Labels(Image):
         int_coord = tuple(np.round(coord).astype(int))
         # If requested fill location is outside data shape then return
         if np.any(np.less(int_coord, 0)) or np.any(
-            np.greater_equal(int_coord, self.shape)
+            np.greater_equal(int_coord, self.data.shape)
         ):
             return
 
@@ -674,7 +702,7 @@ class Labels(Image):
 
         if not (self.n_dimensional or self.ndim == 2):
             # if working with just the slice, update the rest of the raw data
-            self.data[tuple(self.dims.indices)] = labels
+            self.data[tuple(self._slice_indices)] = labels
 
         if refresh is True:
             self.refresh()
@@ -712,11 +740,13 @@ class Labels(Image):
                     ),
                     1,
                 )
-                for c, s, brush_size in zip(coord, self.shape, brush_size_dims)
+                for c, s, brush_size in zip(
+                    coord, self.data.shape, brush_size_dims
+                )
             )
         elif self.brush_shape == "circle":
             slice_coord = [int(np.round(c)) for c in coord]
-            shape = self.shape
+            shape = self.data.shape
             if not self.n_dimensional and self.ndim > 2:
                 coord = [coord[i] for i in self.dims.displayed]
                 shape = [shape[i] for i in self.dims.displayed]
