@@ -1,4 +1,6 @@
 import numpy as np
+
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Surface
 
 
@@ -32,7 +34,7 @@ def test_random_3D_surface():
     assert layer._data_view.shape[1] == 2
     assert layer._view_vertex_values.ndim == 1
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
     assert layer._view_vertex_values.ndim == 1
 
@@ -50,7 +52,7 @@ def test_random_4D_surface():
     assert layer._data_view.shape[1] == 2
     assert layer._view_vertex_values.ndim == 1
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
     assert layer._view_vertex_values.ndim == 1
 
@@ -69,13 +71,13 @@ def test_random_3D_timeseries_surface():
     assert layer._view_vertex_values.ndim == 1
     assert layer.shape[0] == 22
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
     assert layer._view_vertex_values.ndim == 1
 
     # If a values axis is made to be a displayed axis then no data should be
     # shown
-    layer.dims.order = [3, 0, 1, 2]
+    layer._slice_dims(ndisplay=3, order=[3, 0, 1, 2])
     assert len(layer._data_view) == 0
 
 
@@ -94,7 +96,7 @@ def test_random_3D_multitimeseries_surface():
     assert layer.shape[0] == 16
     assert layer.shape[1] == 22
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
     assert layer._view_vertex_values.ndim == 1
 
@@ -137,3 +139,13 @@ def test_surface_gamma():
     # Set gamma as keyword argument
     layer = Surface(data, gamma=gamma)
     assert layer.gamma == gamma
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    data = [(-5, 0), (0, 15), (30, 12)]
+    min_val = (-5, 0)
+    max_val = (30, 15)
+    layer = Surface((np.array(data), np.array((0, 1, 2)), np.array((0, 0, 0))))
+    extent = np.array((min_val, max_val))
+    check_layer_world_data_extent(layer, extent, (3, 1), (20, 5))
