@@ -4,15 +4,54 @@ import collections.abc
 import inspect
 import itertools
 import re
-
 from enum import Enum, EnumMeta
 from os import PathLike, fspath, path
 from typing import Optional, Sequence, Type, TypeVar
+from urllib.parse import urlparse
 
 import numpy as np
 
 ROOT_DIR = path.dirname(path.dirname(__file__))
 
+try:
+    from importlib import metadata as importlib_metadata
+except ImportError:
+    import importlib_metadata  # type: ignore
+
+
+def running_as_bundled_app() -> bool:
+    """Infer whether we are running as a briefcase bundle"""
+    # https://github.com/beeware/briefcase/issues/412
+    # https://github.com/beeware/briefcase/pull/425
+    # this assumes the name of the app stays "napari"
+    try:
+        return importlib_metadata.metadata("napari").get("App-ID") is not None
+    except importlib_metadata.PackageNotFoundError:
+        """When bundled in another app. Return false to not conflict with napari bundle"""
+        return False
+
+
+def in_jupyter() -> bool:
+    """Return true if we're running in jupyter notebook/lab or qtconsole."""
+    try:
+        from IPython import get_ipython
+
+        return get_ipython().__class__.__name__ == 'ZMQInteractiveShell'
+    except Exception:
+        pass
+    return False
+
+
+def in_ipython() -> bool:
+    """Return true if we're running in an IPython interactive shell."""
+    try:
+        from IPython import get_ipython
+
+        return get_ipython().__class__.__name__ == 'TerminalInteractiveShell'
+    except Exception:
+        pass
+    return False
+
 
 def str_to_rgb(arg):
     """Convert an rgb string 'rgb(x,y,z)' to a list of ints [x,y,z].
@@ -182,6 +221,9 @@ class StringEnum(Enum, metaclass=StringEnumMeta):
 
 
 camel_to_snake_pattern = re.compile(r'(.)([A-Z][a-z]+)')
+camel_to_spaces_pattern = re.compile(
+    r"((?<=[a-z])[A-Z]|(?<!\A)[A-R,T-Z](?=[a-z]))"
+)
 
 
 def camel_to_snake(name):
@@ -189,6 +231,10 @@ def camel_to_snake(name):
     return camel_to_snake_pattern.sub(r'\1_\2', name).lower()
 
 
+def camel_to_spaces(val):
+    return camel_to_spaces_pattern.sub(r" \1", val)
+
+
 T = TypeVar('T', str, Sequence[str])
 
 
@@ -214,7 +260,8 @@ def abspath_or_url(relpath: T) -> T:
 
     if isinstance(relpath, (str, PathLike)):
         relpath = fspath(relpath)
-        if relpath.startswith(('http:', 'https:', 'ftp:', 'file:')):
+        urlp = urlparse(relpath)
+        if urlp.scheme and urlp.netloc:
             return relpath
         return path.abspath(path.expanduser(relpath))
 
